/******************************************************************************
*       SOFA, Simulation Open-Framework Architecture, version 1.0 beta 4      *
*                (c) 2006-2009 MGH, INRIA, USTL, UJF, CNRS                    *
*                                                                             *
* This library is free software; you can redistribute it and/or modify it     *
* under the terms of the GNU Lesser General Public License as published by    *
* the Free Software Foundation; either version 2.1 of the License, or (at     *
* your option) any later version.                                             *
*                                                                             *
* This library is distributed in the hope that it will be useful, but WITHOUT *
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or       *
* FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License *
* for more details.                                                           *
*                                                                             *
* You should have received a copy of the GNU Lesser General Public License    *
* along with this library; if not, write to the Free Software Foundation,     *
* Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301 USA.          *
*******************************************************************************
*                               SOFA :: Modules                               *
*                                                                             *
* Authors: The SOFA Team and external contributors (see Authors.txt)          *
*                                                                             *
* Contact information: contact@sofa-framework.org                             *
******************************************************************************/
#ifndef SOFA_COMPONENT_ODESOLVER_EULERSOLVER_H
#define SOFA_COMPONENT_ODESOLVER_EULERSOLVER_H

#include <sofa/simulation/common/MechanicalVisitor.h>
#include <sofa/core/componentmodel/behavior/OdeSolver.h>
#include <sofa/component/odesolver/OdeSolverImpl.h>
#include <sofa/helper/map.h>

namespace sofa
{

  namespace component
  {

    namespace odesolver
    {
      using namespace sofa::component::linearsolver;
      /** The simplest time integration.
	  Two variants are available, depending on the value of field "symplectic".
	  If true (the default), the symplectic variant of Euler's method is applied:
	  If false, the basic Euler's method is applied (less robust)
      */
      class SOFA_COMPONENT_ODESOLVER_API EulerSolver : public OdeSolverImpl
	{
	public:

	  EulerSolver();
	  void solve (double dt);


	  Data<bool> symplectic;    

	  /// Given an input derivative order (0 for position, 1 for velocity, 2 for acceleration),
	  /// how much will it affect the output derivative of the given order.
	  virtual double getIntegrationFactor(int inputDerivative, int outputDerivative) const
	  {
	    const double dt = getContext()->getDt();
	    double matrix[3][3] = {
	      { 1, dt, ((symplectic.getValue())?dt*dt:0.0)},
	      { 0, 1, dt},
	      { 0, 0, 0}};
	    if (inputDerivative >= 3 || outputDerivative >= 3)
	      return 0;
	    else
	      return matrix[outputDerivative][inputDerivative];
	  }

	  /// Given a solution of the linear system,
	  /// how much will it affect the output derivative of the given order.
	  ///
	  virtual double getSolutionIntegrationFactor(int outputDerivative) const
	  {
	    const double dt = getContext()->getDt();
	    double vect[3] = { ((symplectic.getValue())?dt*dt:0.0), dt, 1};
	    if (outputDerivative >= 3)
	      return 0;
	    else
	      return vect[outputDerivative];
	  }
	  void init()
	  {
	    OdeSolverImpl::init();
	    reinit();
	  }

	};

    } // namespace odesolver

  } // namespace component

} // namespace sofa

#endif
